\documentclass[11pt]{article}

%include polycode.fmt
%if style /= newcode
%format $ = "\mathbin{\textsf{\small\$}}"
%format >>= = "\mathbin{\scriptstyle\rangle\!\rangle\!\!=}"
%format <$> = "\mathbin{\tikz{\node[circle,draw,inner sep=0pt]{\textsf{\scriptsize\$}};}}"
%format `on` = "\mathbin{\mathbf{on}}"
%format d_1
%format d_2
%format d_3
%format d_4

%endif

\usepackage{charter}
\usepackage[charter]{mathdesign}
\usepackage[onehalfspacing]{setspace}
\usepackage[margin=1in,letterpaper]{geometry}
\usepackage{tikz}

\setlength{\blanklineskip}{0.5\baselineskip}

\begin{document}

%if style == newcode

This is the module header needed to compile the module, but which would only
clutter the description of the algorithm.
So we hide it from latex.

> module Algos.DS.Stack ( Stack
>                       , emptyStack
>                       , stackFromList
>                       , listFromStack
>                       , push
>                       , pop
>                       , top
>                       ) where

%endif

\title{Part 7\\---\\A Stack}
\author{CSCI 3110 Code}
\date{Summer 2015}
\maketitle
\thispagestyle{empty}

\noindent A stack is of course easy to implement as a list.  Pushing an element
prepends the element to the front of the list.  Popping an element removes the
frontmost element from the stack.  To hide the implementation details, we wrap
the list in a newtype:

> newtype Stack t = Stack [t]

We want to be able to convert back and forth between a stack and a list:

> stackFromList  ::  [t] -> Stack t
> stackFromList  =   Stack
>
> listFromStack             ::  Stack t -> [t]
> listFromStack (Stack xs)  =   xs

An empty stack can now be created from an empty list:

> emptyStack  ::  Stack t
> emptyStack  =   stackFromList []

Finally, the basic stack operations: pushing, popping, and inspecting the top of
the stack:

> push                ::  Stack t -> t -> Stack t
> push (Stack xs)  x  =   Stack (x:xs)
>
> pop                  ::  Stack t -> Stack t
> pop (Stack (_:xs))   =   Stack xs
>
> top                ::  Stack t -> Maybe t
> top (Stack [])     =   Nothing
> top (Stack (x:_))  =   Just x

\end{document}